import { HeytingAlgebra } from './HeytingAlgebra';
/**
 * Boolean algebras are Heyting algebras with the additional constraint that the law of the excluded middle is true
 * (equivalently, double-negation is true).
 *
 * Instances should satisfy the following laws in addition to the {@link HeytingAlgebra} laws:
 *
 * - Excluded middle: `a ∨ ¬a = 1`
 *
 * Boolean algebras generalize classical logic: one is equivalent to "true" and zero is equivalent to "false".
 *
 * @typeclass
 * @since 1.4.0
 */
export interface BooleanAlgebra<A> extends HeytingAlgebra<A> {
}
/**
 * @instance
 * @since 1.4.0
 */
export declare const booleanAlgebraBoolean: BooleanAlgebra<boolean>;
/**
 * @instance
 * @since 1.4.0
 */
export declare const booleanAlgebraVoid: BooleanAlgebra<void>;
/**
 * @function
 * @since 1.4.0
 */
export declare const getFunctionBooleanAlgebra: <B>(B: BooleanAlgebra<B>) => <A = never>() => BooleanAlgebra<(a: A) => B>;
/**
 * Every boolean algebras has a dual algebra, which involves reversing one/zero as well as join/meet.
 * @function
 * @since 1.4.0
 */
export declare const getDualBooleanAlgebra: <A>(B: BooleanAlgebra<A>) => BooleanAlgebra<A>;
